/***************************************************************************
                          LMSensors.cpp  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>

#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include <qstrlist.h>
#include <qobjectlist.h>

#include "lmsensors.h"

//****************************************************************************
// Public methods
//****************************************************************************

LMSensors::LMSensors(KConfig *cfg, QObject *parent, const char * name)
                                  : QObject(parent,name)
{
 ksConfig      = cfg;
 tempScale     = LMSensor::dgCelsius;
 updateInterval= 5;
 if(initSensors()) createSensors();
 readConfig();
}

LMSensors::~LMSensors()
{
 if(count()) sensors_cleanup();
}

bool LMSensors::initSensors()
{
 FILE *fp=fopen("/etc/sensors.conf","r");
 if(!fp) {
   qWarning("KSensors error: /etc/sensors.conf not found !");
   return false;
 }
 int err= sensors_init(fp);
 if(err) {
   qWarning("KSensors error: sensors_init fail, error code %d",err);
   return false;
 }
 fclose(fp);
 return true;
}

void LMSensors::createSensors()
{
 createLMSensors();
 createHDSensors();
}

void LMSensors::createLMSensors()
{
 LMSensorsChip *chip;
 const sensors_chip_name *chip_name;
 int err= 0;
 while( (chip_name= sensors_get_detected_chips(&err)) )
   if( existSensor(chip_name,"temp")  ||
       existSensor(chip_name,"temp1") ||
       existSensor(chip_name,"fan")   ||
       existSensor(chip_name,"fan1")    )
       {
          chip= new LMSensorsChip(ksConfig,chip_name,this);
       }
}

void LMSensors::createHDSensors()
{
 HDSensorsList *disks= new HDSensorsList(ksConfig,this,"Disks");
 if(disks->count()==0) delete disks;
}


int LMSensors::existSensor(const sensors_chip_name *chip_name,const char *sensor_name)
{
int nr1,nr2;
const sensors_feature_data *sensor_data;

 nr1=nr2= 0;
 while( (sensor_data= sensors_get_all_features(*chip_name, &nr1, &nr2)) )
   if( !strcmp(sensor_data->name,sensor_name) )
     return sensor_data->number;
 return 0;
}

void LMSensors::setMonitorized(bool enable)
{
 QObjectList *list= getSensorsChips();
 if(list)
   for(LMSensorsChip *sensor=(LMSensorsChip*)list->first(); sensor!=0; sensor= (LMSensorsChip *)list->next())
     sensor->setMonitorized(enable);
}

void LMSensors::setUpdateInterval(int seconds)
{
 QObjectList *list= getSensorsChips();
 if(list)
   for(LMSensorsChip *sensor=(LMSensorsChip*)list->first(); sensor!=0; sensor= (LMSensorsChip *)list->next())
     sensor->setUpdateInterval(seconds);
 updateInterval= seconds;
}

void LMSensors::setTempScale(LMSensor::TempScale scale)
{
 if(tempScale==scale) return;
 tempScale= scale;

 QObjectList *list= getSensorsChips();
 if(list)
   for(LMSensorsChip *sensor=(LMSensorsChip*)list->first(); sensor!=0; sensor= (LMSensorsChip *)list->next())
     sensor->setTempScale(scale);
}


void LMSensors::readConfig()
{
  if(ksConfig) {
    ksConfig->setGroup( "Sensors" );
    setUpdateInterval(ksConfig->readNumEntry("UpdateInterval",5));
    QString str= ksConfig->readEntry("Scale" ,"C");
    LMSensor::TempScale tempScale;
    if(str=="F") tempScale= LMSensor::dgFahrenheit;
    else if(str=="K") tempScale= LMSensor::dgKelvin;
         else             tempScale= LMSensor::dgCelsius;
    setTempScale(tempScale);
  }
}

Sensor *LMSensors::getSensor(const char *name)
{
 int        index= count();
 Sensor *sensor= 0;
 while(--index>=0 && !sensor) {
    sensor= getSensorsChip(index)->getSensor(name);
 }
 return sensor;
}

void LMSensors::writeConfig()
{
  if(ksConfig) {
    ksConfig->setGroup( "Sensors" );
    ksConfig->writeEntry("UpdateInterval",updateInterval);
    QString str;
    switch(tempScale) {
     case LMSensor::dgCelsius   : str='C'; break;
     case LMSensor::dgKelvin    : str='K'; break;
     case LMSensor::dgFahrenheit: str='F'; break;
    }
    ksConfig->writeEntry("Scale" ,str);
  }
}

void LMSensors::childEvent( QChildEvent *e )
{
 if(e->inserted()) {
    connect((SensorsList *)e->child(),SIGNAL(valueChanged(Sensor *)) ,this,SIGNAL(valueChanged(Sensor  *)));
    connect((SensorsList *)e->child(),SIGNAL(configChanged(Sensor *)),this,SIGNAL(configChanged(Sensor *)));
 }
}


/*********************************************************************************/
