/***************************************************************************
                          LMSensors.h  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LMSENSORS_H
#define LMSENSORS_H

#include <qobject.h>
#include <qobjcoll.h>
#include <kconfig.h>
#include <qstringlist.h>

#include <sensors/sensors.h>

#include "lmsensorschip.h"
#include "hdsensorslist.h"

/**
  *@author Michael
  */

class LMSensors : public QObject
{
Q_OBJECT

  friend class LMSensor;

public:

	LMSensors(KConfig *cfg=0, QObject *parent=0, const char *name=0);
	~LMSensors();

  void setMonitorized(bool enable);
  QObjectList *getSensorsChips()                  { return (QObjectList *)children(); };
  LMSensorsChip *getSensorsChip(int index)        { return children() ? (LMSensorsChip *)((QObjectList *)children())->at(index) : 0; }
  LMSensorsChip *getSensorsChip(const char *name) { return (LMSensorsChip *)child(name); }
  int count() 									                  { return children() ? children()->count() : 0; }
  Sensor *getSensor(const char *name);

  void  setTempScale(LMSensor::TempScale scale);
  LMSensor::TempScale  getTempScale()                { return tempScale; }

  void setUpdateInterval(int seconds);
  int  getUpdateInterval()                {  return updateInterval; }

  KConfig *getConfig()                    { return ksConfig;  }

  void readConfig();
  void writeConfig();

signals:

  void valueChanged(Sensor *);
  void configChanged(Sensor *);

private:

  int  updateInterval;
  LMSensor::TempScale tempScale;
  KConfig *ksConfig;

  bool initSensors();
  void createSensors();
  void createLMSensors();
  void createHDSensors();
  void childEvent ( QChildEvent *e );

  static int existSensor(const sensors_chip_name *chip_name,const char *sensor_name);
};

#endif
